
findOptimalSegmentations <- function(y, x, kMax)
{
    x <- as.double(x)
    y <- as.double(y)

    if (length(x) != length(y))
    {
        stop("x and y vectors must be the same length")
    }

    nMax <- length(x)
    kMax <- as.integer(kMax)

    if (!((1 <= kMax) && (kMax <= nMax)))
    {
        stop("kMax must be at least 1 and at most the length of the x and y vectors")
    }

    segSumErrorSquared <- rep(0.0, kMax)
    segBoundaries      <- rep(0L, kMax * (kMax + 1) / 2)

    returnValue <- as.integer(0)

    result <- .C(bridge_FindOptimalSegmentations, x, y, nMax, kMax, segBoundaries, segSumErrorSquared, returnValue)

    names(result) <- c("x", "y", "nMax", "kMax", "segBoundaries", "segSumErrorSquared", "returnValue")

    if (result$returnValue != 0)
    {
        stop("out of memory")
    }

    # return the result as a list of lists.

    lapply(
        1:kMax,
        function (k)
        {
            sumErrorSquared = result$segSumErrorSquared[k]
            segmentation = result$segBoundaries[(k * (k - 1) / 2 + 1):(k * (k + 1) / 2)]

            alphaEstimated <- sapply(
                1:k,
                function (i)
                {
                    left  <- if (i == 1) 1 else segmentation[i - 1] + 1
                    right <- segmentation[i]

                    x.seg <- x[left:right]
                    y.seg <- y[left:right]

                    alpha <- sum(x.seg * y.seg) / sum(x.seg * x.seg)
                }
            )

            sigmaEstimated <- sqrt(sumErrorSquared / nMax)

            ## BIC calculation for this segmentation starts here

            # The BIC calculation as given here is taken from Zhang/Siegmund (2007), Theorem 2.
            # While this works quite fine in practice, some questions remain whether this is
            # theoretically justified.

            SSall <- result$segSumErrorSquared[1]

            sum.log.segment_lengths <- sum(log(diff(c(0, segmentation))))

            bic.term1 <- ((nMax - k + 1) / 2) * log(SSall / sumErrorSquared)
            bic.term2 <- lgamma((nMax - k + 1) / 2) - lgamma((nMax + 1) / 2)
            bic.term3 <- (k/2) * log(SSall)
            bic.term4 <- -sum.log.segment_lengths / 2
            bic.term5 <- (1/2 - k) * log(nMax)

            modifiedBIC <- bic.term1 + bic.term2 + bic.term3 + bic.term4 + bic.term5

            ### BIC calculation for this segmentation ends here

            # construct the list entry for this value ok k.

            list(k = k, sumErrorSquared = sumErrorSquared, sigmaEstimated = sigmaEstimated, modifiedBIC = modifiedBIC, segmentation = segmentation, alphaEstimated = alphaEstimated)
        }
    )
}
