\name{findOptimalSegmentations}

\alias{findOptimalSegmentations}

\title{Find Optimal Segmentations}

\description{
    Calculate optimal segmentation solutions for different segment counts,
    given capture-counts of a test sample and a reference sample.
}

\usage{
    findOptimalSegmentations(y, x, kMax)
}

\arguments{
    \item{y}{the vector of test sample counts.}
    \item{x}{the vector of reference sample counts.}
    \item{kMax}{the maximum number of segments for which optimal an solution is determined.}
}

\value{
    The function will return a list. Each element in the list represents an optimal
    segmentation solution, assuming segmentation in \code{k} segments. The solutions are ordered
    by the value of \code{k}, which runs from 1 to \code{kMax}.

    Each solution is itself a list with six named elements:

    \itemize{
        \item{\code{k} --- the number of segments in this solution.}
        \item{\code{sumErrorSquared} --- the error-squared of this solution, summed over all segments.}
        \item{\code{sigmaEstimated} --- the estimated standard deviation of this solution.}
        \item{\code{modifiedBIC} --- the modified BIC value of this solution.

            This value provides guidance with regard to which of the returned solutions to pick. There is an
            inherent trade-off between too few segments and too many segments; the
            modified BIC value is designed to be maximized for the 'optimal' number of
            segments under a number of assumptions.}
        \item{\code{segmentation} --- the segmentation of this solution.
            It is a vector of \code{k} integers, each representing a segment.
            For each segment, the right-most index is given.}
        \item{\code{alphaEstimated} --- the estimated ratio between \code{y} and \code{x} for each of the segments.}
    }
}

\note{

    \itemize{

    \item The length of vectors \code{x} and \code{y} should be the same.
    \item The value \code{kMax} should be a positive integer that does not exceed the length of \code{x} and \code{y}.
    \item The segmentations returned will be optimal (barring numerical issues). This means that no solutions exist
          with the same \code{k} but lower \code{sumErrorSquared}.
    \item Although it is unlikely to happen with actual data, the optimal solution as returned for a given number
          of segments (\code{k}) is not necessarily unique. There may be other segmentations for a given value of \code{k}
          that have the same minimal value for \code{sumErrorSquared}.
    \item{The modified BIC calculation as used is taken from Zhang/Siegmund (2007), Theorem 2.

          It appears to work well in practice, but it must be noted that the statistical model underlying the
          method implemented here is different from the model for which the modified BIC is designed.}
    }
}

\examples{

library(optimalCaptureSegmentation)

## set up test-case

# We will synthesize capture bait counts for a reference (x) and test (y) sample.

# Our test case comprises a single chromosome with 1000 baits. The
# simulated test sample y has 5 segments:
#
# segment.nr   first.bait.nr   last.bait.nr   length.segment     alpha
# --------------------------------------------------------------------
#     1               1             100           100              1.0
#     2             101             300           200              0.5
#     3             301             400           100              1.0
#     4             401             800           400              0.0
#     5             801            1000           200              3.0
#
# The 'alpha' value denotes the expected ratio between the counts per bait
# found for each bait in the test sample vs. the reference sample.
#
# For example, for each of the two hundred baits region in segment #5, we expect
# to find three times as many DNA fragments in the 'test' sample compared to the
# 'reference' sample.

alphaSimulated <- rep(c(1.0, 0.5, 1.0, 0.0, 3.0), c(100, 200, 100, 400, 200))

num.baits = length(alphaSimulated)

# Simulate read counts for all baits in the reference sample.

set.seed(123456) # make this reproducible

xSimulated <- runif(n = num.baits, min = 100.0, max = 150.0)

# Simulate read counts for all baits in the test sample.
# Note that the error sigma is assumed to be constant over the entire chromosome.

sigmaSimulated <- 17.0

errorSimulated <- rnorm(n = num.baits, mean = 0, sd = sigmaSimulated)

ySimulated <- alphaSimulated * xSimulated + errorSimulated

## solve test-case

# Next, we try to recover the segmentation, alpha, and sigma from the x and y
# vectors given. We will try possible segmentations with up to 10 segments.

solutions <- findOptimalSegmentations(ySimulated, xSimulated, 10)

## show solutions

for (sol in solutions)
{
    cat("k ..................... : ", sol$k                                      , "\n",
        "sumErrorSquared ....... : ", sol$sumErrorSquared                        , "\n",
        "sigmaEstimated ........ : ", sol$sigmaEstimated                         , "\n",
        "modifiedBIC............ : ", sol$modifiedBIC                            , "\n",
        "segmentation .......... : ", paste(sol$segmentation, collapse = ", ")   , "\n",
        "alphaEstimated ........ : ", paste(sol$alphaEstimated, collapse = ", ") , "\n",
        "\n", sep = "")
}

## select and print best solution

# The modifiedBIC value that goes with each solution is ideally maximized when the
# number of segments is correct.

best.solution <- solutions[[which.max(sapply(solutions, function (sol) sol$modifiedBIC))]]

print(best.solution)
}

% \reference{
% }
